/* guestbook.c, boone, 10/28/92
   Telnettable guestbook service 
   Copyright (C) 1993, Trustees of Michigan State University */

/* Modifications:
   10/28/92 Boone      Initial coding
   04/06/93 Boone      Drool-proofed the instructions, added time of call
                       to log file, quit writing responses with no comment.
   End Modifications */

/* Description:
   Wait on a port.  When a connection arrives, accept it and fork().
   Prompt for name, e-mail address, geographical location, and 
   some comments.
   End Description */

#define _BSD    43
#define CLIMIT  10
#define GBPORT  8000
#define LOGFILE "guestbook.log"
#define MAXSTR  256

/* This is really disgusting. */

#define BANNER1 "\r\nThis is Michigan State University's Gopher Guest Book.\r\n"
#define BANNER2 "We use feedback from visitors like yourself to help us\r\n"
#define BANNER3 "improve this service.  We would appreciate your responses\r\n"
#define BANNER4 "to the following questions.  Your responses will be read\r\n"
#define BANNER5 "by MSU Gopher Team staff.  Your originating Internet\r\n"
#define BANNER6 "address will be recorded with your remarks.  Please do not\r\n"
#define BANNER7 "use your cursor keys to edit your comments; only the \r\n"
#define BANNER8 "backspace key will work.\r\n"
#define BANNER9 "\r\nMembers of the MSU community who need help with Gopher\r\n"
#define BANNERA "or who wish to report a problem should contact the\r\n"
#define BANNERB "Computing Information Center at 517-353-1800.  (The CIC is\r\n"
#define BANNERC "located at 305 Computer Center.)  General problems can be\r\n"
#define BANNERD "sent to gopher@gopher.msu.edu.\r\n"

#define NPROMPT "\r\nYour full name? "
#define EPROMPT "Your e-mail address? "
#define OPROMPT "Your organization? "
#define LPROMPT "City, State/Province, Country? "
#define CHEADER "Please enter your comments below, or a blank line to end.\r\n"
#define CLHEADR "You may enter up to %d lines.\r\n"
#define YOUSURE "\r\nOk to save these comments (Y/n)? "
#define THANKS  "\r\nThank you for taking time to comment.\r\n\r\n"
#define DASHES  "---------------\n\n"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/lockf.h>
#include <sys/time.h>

int serversock(int);
void writeline(int, char *, int);

void strip(char *s)

{
	int i;

	for (i = strlen(s)-1; i >= 0; i--)
		if (! isprint(s[i]))
			s[i] = ' ';
	for (i = strlen(s)-1; i >= 0; i--)
	{
		if (s[i] == ' ')
			s[i] = '\0';
		else
			break;
	}
}

int main(int argc, char *argv[])

{
	int i, j;			/* Miscellaneous counters */
	int sfd;			/* Initial socket fd */
	int cfd;			/* Client-connected socket fd */
	int fromlen;			/* Size of client address structure */
	struct sockaddr from;		/* Client address structure */
	int clientlen;			/* Size of peer address structure */
	struct sockaddr_in client;	/* Peer address structure */
	int pid;			/* Process id of child */
	char inbuf[MAXSTR+1];		/* Output buffer */
	char outbuf[MAXSTR+1];		/* Output buffer */
	struct hostent *hp;		/* Pointer to host name structure */
	FILE *log;			/* Log file descriptor */
	char yn;			/* Yes/no work variable */
	time_t now;			/* Time work variable */

	char calltime[32];		/* Time of call string */
	char ndata[MAXSTR+1];		/* Name data */
	char edata[MAXSTR+1];		/* Email data */
	char odata[MAXSTR+1];		/* Organization data */
	char ldata[MAXSTR+1];		/* Location data */
	char cdata[CLIMIT][MAXSTR+1];	/* Comments data */

/* Main wait for connections loop */

	fromlen = sizeof(from);
	sfd = serversock(GBPORT);
	while(1)
	{
		cfd = accept(sfd, &from, &fromlen);
		pid = fork();
		if (! pid)
			break;
		close(cfd);
	}

/* Handle a call */

	clientlen = sizeof(client);
	getpeername(cfd, &client, &clientlen);
	hp = gethostbyaddr(&(client.sin_addr.s_addr),
		sizeof(client.sin_addr.s_addr), AF_INET);
	writeline(cfd, BANNER1, strlen(BANNER1));
	writeline(cfd, BANNER2, strlen(BANNER2));
	writeline(cfd, BANNER3, strlen(BANNER3));
	writeline(cfd, BANNER4, strlen(BANNER4));
	writeline(cfd, BANNER5, strlen(BANNER5));
	writeline(cfd, BANNER6, strlen(BANNER6));
	writeline(cfd, BANNER7, strlen(BANNER7));
	writeline(cfd, BANNER8, strlen(BANNER8));
	writeline(cfd, BANNER9, strlen(BANNER9));
	writeline(cfd, BANNERA, strlen(BANNERA));
	writeline(cfd, BANNERB, strlen(BANNERB));
	writeline(cfd, BANNERC, strlen(BANNERC));
	writeline(cfd, BANNERD, strlen(BANNERD));

	writeline(cfd, NPROMPT, strlen(NPROMPT));
	readline(cfd, ndata, sizeof(ndata));
	strip(ndata);

	writeline(cfd, EPROMPT, strlen(EPROMPT));
	readline(cfd, edata, sizeof(edata));
	strip(edata);

	writeline(cfd, OPROMPT, strlen(OPROMPT));
	readline(cfd, odata, sizeof(odata));
	strip(odata);

	writeline(cfd, LPROMPT, strlen(LPROMPT));
	readline(cfd, ldata, sizeof(ldata));
	strip(ldata);

	i = 0;
	writeline(cfd, CHEADER, strlen(CHEADER));
	sprintf(outbuf, CLHEADR, CLIMIT);
	writeline(cfd, outbuf, strlen(outbuf));
	while(1)
	{
		if (i >= CLIMIT) break;
		sprintf(outbuf, "%2d> ", i+1);
		writeline(cfd, outbuf, strlen(outbuf));
		readline(cfd, inbuf, sizeof(inbuf));
		strip(inbuf);
		if (strlen(inbuf) > 0)
			strncpy(cdata[i], inbuf, MAXSTR);
		else
			break;
		i++;
	}
	writeline(cfd, YOUSURE, strlen(YOUSURE));
	readline(cfd, inbuf, sizeof(inbuf));
	strip(inbuf);
	if (strlen(inbuf) == 0)
		inbuf[0] = 'Y';
	yn = tolower(inbuf[0]);

	writeline(cfd, THANKS, strlen(THANKS));

/* Write data to log file */

	if ((yn != 'n') && (i > 0))
	{
		if ((log = fopen(LOGFILE, "a")) == NULL)
		{
			fputs("unable to open log file\n", stderr);
			exit(1);
		}
		lockf(fileno(log), F_LOCK, 0);
		lseek(fileno(log), 0L, SEEK_END);
		if (hp == NULL)
		{
			sprintf(outbuf, "gethostbyaddr returned %d\n", h_errno);
			write(fileno(log), outbuf, strlen(outbuf));
			sprintf(outbuf, "Call from %d.%d.%d.%d\n", 
				from.sa_data[2], from.sa_data[3], 
				from.sa_data[4], from.sa_data[5]);
		}
		else
		{
			sprintf(outbuf, "Call from %s\n", hp->h_name);
			write(fileno(log), outbuf, strlen(outbuf));
		}
		now = time(NULL);
		sprintf(outbuf, "Time: %s", asctime(localtime(&now)));
		write(fileno(log), outbuf, strlen(outbuf));
		sprintf(outbuf, "Name: %s\n", ndata);
		write(fileno(log), outbuf, strlen(outbuf));
		sprintf(outbuf, "Email: %s\n", edata);
		write(fileno(log), outbuf, strlen(outbuf));
		sprintf(outbuf, "Organization: %s\n", odata);
		write(fileno(log), outbuf, strlen(outbuf));
		sprintf(outbuf, "Location: %s\n", ldata);
		write(fileno(log), outbuf, strlen(outbuf));
		for (j = 0; j < i; j++)
		{
			sprintf(outbuf, "> %s\n", cdata[j]);
			write(fileno(log), outbuf, strlen(outbuf));
		}
		write(fileno(log), DASHES, strlen(DASHES));
		lockf(fileno(log), F_ULOCK, 0);
		fclose(log);
	}

/* Done handling call */

	sleep(10);
	close(cfd);
	exit(0);
}
